@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: gadgets.c,v 1.115.2.6 2016/07/08 20:48:34 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: gadgets.c,v 1.115.2.9 2017/02/25 05:20:32 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - gadgets.c */
@@ -58,7 +58,7 @@ legend_key keyT = DEFAULT_KEY_PROPS;
 color_box_struct color_box; /* initialized in init_color() */
 color_box_struct default_color_box = {SMCOLOR_BOX_DEFAULT, 'v', 1, LT_BLACK, LAYER_FRONT, 0,
 					{screen, screen, screen, 0.90, 0.2, 0.0},
-					{screen, screen, screen, 0.05, 0.6, 0.0},
+					{screen, screen, screen, 0.05, 0.6, 0.0}, FALSE,
 					{0,0,0,0} };
 
 /* The graph box, in terminal coordinates, as calculated by boundary()
@@ -134,6 +134,9 @@ double zero = ZERO;
 double pointsize = 1.0;
 double pointintervalbox = 1.0;
 
+/* used for filled points */
+t_colorspec background_fill = BACKGROUND_COLORSPEC;
+
 /* set border */
 int draw_border = 31;	/* The current settings */
 int user_border = 31;	/* What the user last set explicitly */
@@ -653,7 +656,9 @@ reset_textcolor(const struct t_colorspec *tc, const struct termentry *t)
 void
 default_arrow_style(struct arrow_style_type *arrow)
 {
-    static const struct lp_style_type tmp_lp_style = DEFAULT_LP_STYLE_TYPE;
+    static const struct lp_style_type tmp_lp_style = 
+	{0, LT_DEFAULT, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0,
+	DEFAULT_COLORSPEC, DEFAULT_DASHPATTERN};
 
     arrow->tag = -1;
     arrow->layer = LAYER_BACK;
