@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: plot3d.c,v 1.231.2.7 2016/06/18 05:59:25 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: plot3d.c,v 1.231.2.9 2016/12/04 00:29:01 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - plot3d.c */
@@ -748,10 +748,13 @@ get_3ddata(struct surface_points *this_plot)
     int xdatum = 0;
     int ydatum = 0;
     int j;
-    double v[MAXDATACOLS];
     int pt_in_iso_crv = 0;
     struct iso_curve *this_iso;
     int retval = 0;
+    double v[MAXDATACOLS];
+
+    /* Initialize the space that will hold input data values */
+    memset(v, 0, sizeof(v));
 
     if (mapping3d == MAP3D_CARTESIAN) {
 	/* do this check only, if we have PM3D / PM3D-COLUMN not compiled in */
@@ -1297,7 +1300,6 @@ eval_3dplots()
     int i;
     struct surface_points **tp_3d_ptr;
     int start_token=0, end_token;
-    int highest_iteration = 0;	/* last index reached in iteration [i=start:*] */
     TBOOLEAN eof_during_iteration = FALSE;  /* set when for [n=start:*] hits NODATA */
     int begin_token;
     TBOOLEAN some_data_files = FALSE, some_functions = FALSE;
@@ -1933,7 +1935,6 @@ eval_3dplots()
 	    if (empty_iteration(plot_iterator))
 		this_plot->plot_type = NODATA;
 	    if (forever_iteration(plot_iterator) && (this_plot->plot_type == NODATA)) {
-		highest_iteration = plot_iterator->iteration;
 		eof_during_iteration = TRUE;
 	    }
 	    if (forever_iteration(plot_iterator) && (this_plot->plot_type == FUNC3D)) {
@@ -1955,7 +1956,6 @@ eval_3dplots()
 	    /* Nothing to do */ ;
 	} else if (next_iteration(plot_iterator)) {
 	    c_token = start_token;
-	    highest_iteration = plot_iterator->iteration;
 	    continue;
 	}
 
@@ -2138,10 +2138,8 @@ eval_3dplots()
 
 	    /* Iterate-over-plot mechanism */
 	    if (crnt_param == 0 && next_iteration(plot_iterator)) {
-		if (plot_iterator->iteration <= highest_iteration) {
-		    c_token = start_token;
-		    continue;
-		}
+		c_token = start_token;
+		continue;
 	    }
 
 	    if (crnt_param == 0)
