@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: tables.c,v 1.110.2.2 2014/01/31 05:22:06 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: tables.c,v 1.133.2.7 2016/10/21 21:00:08 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - tables.c */
@@ -37,6 +37,7 @@ static char *RCSid() { return RCSid("$Id: tables.c,v 1.110.2.2 2014/01/31 05:22:
 #include "tables.h"
 
 #include "command.h"
+#include "datablock.h"
 #include "fit.h"
 #include "setshow.h"
 #include "term_api.h"
@@ -66,16 +67,16 @@ const struct gen_ftable command_ftbl[] =
     { "?", help_command },
     { "hi$story", history_command },
     { "if", if_command },
+    { "import", import_command },
     { "else", else_command },
     { "l$oad", load_command },
     { "pa$use", pause_command },
     { "p$lot", plot_command },
     { "pr$int", print_command },
+    { "printerr$or", printerr_command },
     { "pwd", pwd_command },
     { "q$uit", exit_command },
-#ifdef VOLATILE_REFRESH
     { "ref$resh", refresh_command },
-#endif
     { "rep$lot", replot_command },
     { "re$read", reread_command },
     { "res$et", reset_command },
@@ -95,6 +96,7 @@ const struct gen_ftable command_ftbl[] =
     { "{", begin_clause },
     { "}", end_clause },
     { ";", null_command },
+    { "$", datablock_command },
     /* last key must be NULL */
     { NULL, invalid_command }
 };
@@ -164,8 +166,11 @@ const struct gen_table set_tbl[] =
     { "box$width", S_BOXWIDTH },
     { "cl$abel", S_CLABEL },
     { "c$lip", S_CLIP },
-    { "cn$trparam", S_CNTRPARAM },
-    { "co$ntours", S_CONTOUR },
+    { "cntrp$aram", S_CNTRPARAM },
+    { "cntrl$abel", S_CNTRLABEL },
+    { "cont$ours", S_CONTOUR },
+    { "dasht$ype", S_DASHTYPE },
+    { "dt", S_DASHTYPE },
     { "da$ta", S_DATA },
 
     { "data$file", S_DATAFILE },
@@ -181,11 +186,13 @@ const struct gen_table set_tbl[] =
     { "fu$nctions", S_FUNCTIONS },
     { "g$rid", S_GRID },
     { "hid$den3d", S_HIDDEN3D },
-    { "his$torysize", S_HISTORYSIZE },
+    { "historysize", S_HISTORYSIZE },	/* Deprecated */
+    { "his$tory", S_HISTORY },
     { "is$osamples", S_ISOSAMPLES },
     { "k$ey", S_KEY },
-    { "keyt$itle", S_KEYTITLE },
+    { "keyt$itle", S_KEY },
     { "la$bel", S_LABEL },
+    { "link", S_LINK },
     { "lines$tyle", S_LINESTYLE },
     { "linetype$s", S_LINETYPE },
     { "ls", S_LINESTYLE },
@@ -193,21 +200,22 @@ const struct gen_table set_tbl[] =
     { "loa$dpath", S_LOADPATH },
     { "loc$ale", S_LOCALE },
     { "log$scale", S_LOGSCALE },
-#ifdef GP_MACROS
     { "mac$ros", S_MACROS },
-#endif
     { "map$ping", S_MAPPING },
     { "map$ping3d", S_MAPPING },
 
-    { "mar$gin", S_MARGIN },
+    { "mar$gins", S_MARGIN },
     { "lmar$gin", S_LMARGIN },
     { "rmar$gin", S_RMARGIN },
     { "tmar$gin", S_TMARGIN },
     { "bmar$gin", S_BMARGIN },
 
+    { "micro", S_MICRO },
+    { "minus$sign", S_MINUS_SIGN },
 #ifdef USE_MOUSE
     { "mo$use", S_MOUSE },
 #endif
+    { "mono$chrome", S_MONOCHROME },
     { "multi$plot", S_MULTIPLOT },
 
     { "mxt$ics", S_MXTICS },
@@ -220,6 +228,8 @@ const struct gen_table set_tbl[] =
     { "nomy2t$ics", S_NOMY2TICS },
     { "mzt$ics", S_MZTICS },
     { "nomzt$ics", S_NOMZTICS },
+    { "mrt$ics", S_MRTICS },
+    { "nomrt$ics", S_NOMRTICS },
     { "mcbt$ics", S_MCBTICS },
     { "nomcbt$ics", S_NOMCBTICS },
     { "of$fsets", S_OFFSETS },
@@ -228,9 +238,10 @@ const struct gen_table set_tbl[] =
     { "pa$rametric", S_PARAMETRIC },
     { "pm$3d", S_PM3D },
     { "pal$ette", S_PALETTE },
+    { "color", S_COLOR },
     { "colorb$ox", S_COLORBOX },
     { "colorn$ames", S_COLORNAMES },
-    { "colors", S_COLORNAMES },
+    { "colors$equence", S_COLORSEQUENCE },
     { "p$lot", S_PLOT },
     { "pointint$ervalbox", S_POINTINTERVALBOX },
     { "poi$ntsize", S_POINTSIZE },
@@ -244,7 +255,7 @@ const struct gen_table set_tbl[] =
     { "su$rface", S_SURFACE },
     { "table", S_TABLE },
     { "t$erminal", S_TERMINAL },
-    { "termo$ptions", S_TERMOPTIONS },
+    { "termopt$ions", S_TERMOPTIONS },
     { "ti$cs", S_TICS },
     { "ticsc$ale", S_TICSCALE },
     { "ticsl$evel", S_TICSLEVEL },
@@ -329,6 +340,7 @@ const struct gen_table set_tbl[] =
     { "zzeroa$xis", S_ZZEROAXIS },
     { "zeroa$xis", S_ZEROAXIS },
     { "rax$is", S_RAXIS },
+    { "paxis", S_PAXIS },
 
     { "z$ero", S_ZERO },
     { NULL, S_INVALID }
@@ -410,7 +422,6 @@ const struct gen_table test_tbl[] =
 {
     { "term$inal", TEST_TERMINAL },
     { "pal$ette", TEST_PALETTE },
-    { "time", TEST_TIME },
     { NULL, TEST_INVALID }
 };
 
@@ -427,6 +438,8 @@ const struct gen_table set_colorbox_tbl[] =
     { "nobo$rder",	S_COLORBOX_NOBORDER },
     { "o$rigin",	S_COLORBOX_ORIGIN },
     { "s$ize",		S_COLORBOX_SIZE },
+    { "inv$ert",	S_COLORBOX_INVERT },
+    { "noinv$ert",	S_COLORBOX_NOINVERT },
     { "fr$ont",		S_COLORBOX_FRONT },
     { "ba$ck",		S_COLORBOX_BACK },
     { NULL, S_COLORBOX_INVALID }
@@ -437,8 +450,8 @@ const struct gen_table set_palette_tbl[] =
 {
     { "pos$itive",	S_PALETTE_POSITIVE },
     { "neg$ative",	S_PALETTE_NEGATIVE },
-    { "gray",		S_PALETTE_GRAY },
-    { "grey",		S_PALETTE_GRAY },
+    { "gray$scale",	S_PALETTE_GRAY },
+    { "grey$scale",	S_PALETTE_GRAY },
     { "col$or",		S_PALETTE_COLOR },
     { "rgb$formulae",	S_PALETTE_RGBFORMULAE },
     { "def$ined",       S_PALETTE_DEFINED },
@@ -479,6 +492,8 @@ const struct gen_table set_pm3d_tbl[] =
     { "clip1$in",	S_PM3D_CLIP_1IN },
     { "clip4$in",	S_PM3D_CLIP_4IN },
     { "map", 		S_PM3D_MAP },
+    { "bo$rder",	S_PM3D_BORDER },
+    { "nobo$rder",	S_PM3D_NOBORDER },
     { "hi$dden3d",	S_PM3D_HIDDEN },
     { "nohi$dden3d",	S_PM3D_NOHIDDEN },
     { "so$lid",		S_PM3D_SOLID },
@@ -625,7 +640,7 @@ struct gen_table default_color_names_tbl[] =
 struct gen_table *pm3d_color_names_tbl = default_color_names_tbl;
 struct gen_table *user_color_names_tbl = NULL;
 const int num_predefined_colors = sizeof(default_color_names_tbl)
-				/ sizeof(struct gen_table);
+				/ sizeof(struct gen_table) - 1;
 int num_userdefined_colors = 0;
 
 
@@ -643,6 +658,10 @@ const struct gen_table show_style_tbl[] =
     { "ell$ipse", SHOW_STYLE_ELLIPSE },
     { "rect$angle", SHOW_STYLE_RECTANGLE },
     { "boxplot", SHOW_STYLE_BOXPLOT },
+    { "parallel$axis", SHOW_STYLE_PARALLEL },
+#ifdef EAM_BOXED_TEXT
+    { "textbox", SHOW_STYLE_TEXTBOX },
+#endif
     { NULL, SHOW_STYLE_INVALID }
 };
 
@@ -684,13 +703,16 @@ const struct gen_table plotstyle_tbl[] =
     { "cir$cles", CIRCLES },
     { "ell$ipses", ELLIPSES },
 #endif
-    { NULL, -1 }
+    { "sur$face", SURFACEGRID },
+    { "parallel$axes", PARALLELPLOT },
+    { "table", TABLESTYLE },
+    { NULL, PLOT_STYLE_NONE }
 };
 
 const struct gen_table filledcurves_opts_tbl[] =
 {
     { "c$losed", FILLEDCURVES_CLOSED },
-    { "x1", FILLEDCURVES_X1 },
+    { "x$1", FILLEDCURVES_X1 },
     { "y1", FILLEDCURVES_Y1 },
     { "x2", FILLEDCURVES_X2 },
     { "y2", FILLEDCURVES_Y2 },
@@ -698,6 +720,7 @@ const struct gen_table filledcurves_opts_tbl[] =
     { "r", FILLEDCURVES_ATR },
     { "above", FILLEDCURVES_ABOVE },
     { "below", FILLEDCURVES_BELOW },
+    { "y",  FILLEDCURVES_Y1 },
     { NULL, -1 }
 };
 
