@@ -1,5 +1,5 @@
 /*
- *$Id: wtext.c,v 1.37.2.6 2014/07/04 07:45:44 markisch Exp $
+ * $Id: wtext.c,v 1.51.2.5 2017/03/04 08:20:41 markisch Exp $
  */
 
 /* GNUPLOT - win/wtext.c */
@@ -42,12 +42,7 @@
 /* WARNING: Do not write to stdout/stderr with functions not listed
    in win/wtext.h */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
-#define STRICT
-
+#include "syscfg.h"
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -57,20 +52,19 @@
 #endif
 #include <sys/stat.h>
 
-/* needed for mouse scroll wheel support */
-#define _WIN32_WINNT 0x0400
-#define _WIN32_IE 0x0501
+#define STRICT
 #include <windows.h>
 #include <windowsx.h>
 #include <commdlg.h>
 #include <commctrl.h>
+#include <tchar.h>
 
 #include "wgnuplib.h"
 #include "winmain.h"
 #include "wresourc.h"
 #include "wcommon.h"
 #include "stdfn.h"
-#include "fit.h"
+#include "plot.h"
 
 /* font stuff */
 #define TEXTFONTSIZE 9
@@ -265,7 +259,7 @@ TextInit(LPTW lptw)
 	SetWindowPos(lptw->hWndText, (HWND)NULL, 0, 0,
 			rect.right, rect.bottom - lptw->StatusHeight,
 			SWP_NOZORDER | SWP_NOACTIVATE);
-	ShowWindow(lptw->hStatusbar, TRUE);
+	ShowWindow(lptw->hStatusbar, SW_SHOW);
     }
 
     lptw->hPopMenu = CreatePopupMenu();
@@ -415,8 +409,8 @@ NewLine(LPTW lptw)
 
     UpdateCaretPos(lptw);
     if (lptw->bFocus && lptw->bGetCh) {
-		UpdateCaretPos(lptw);
-		ShowCaret(lptw->hWndText);
+	UpdateCaretPos(lptw);
+	ShowCaret(lptw->hWndText);
     }
 
     if (lptw->CursorFlag)
@@ -429,6 +423,7 @@ static void
 UpdateScrollBars(LPTW lptw)
 {
     signed int length;  /* this must be signed for this to work! */
+    SCROLLINFO si;
 
     /* horizontal scroll bar */
     length = sb_max_line_length(&(lptw->ScreenBuffer)) + 1;
@@ -436,8 +431,17 @@ UpdateScrollBars(LPTW lptw)
 	/* maximum horizontal scroll position is given by maximum line length */
 	lptw->ScrollMax.x = max(0, lptw->CharSize.x * length - lptw->ClientSize.x);
 	lptw->ScrollPos.x = min(lptw->ScrollPos.x, lptw->ScrollMax.x);
-	SetScrollRange(lptw->hWndText, SB_HORZ, 0, lptw->ScrollMax.x, FALSE);
-	SetScrollPos(lptw->hWndText, SB_HORZ, lptw->ScrollPos.x, TRUE);
+
+	/* update scroll bar page size, range and position */
+	si.cbSize = sizeof(SCROLLINFO);
+	si.fMask = SIF_PAGE | SIF_RANGE | SIF_POS;
+	si.nPage = lptw->ClientSize.x;
+	si.nMin = 0;
+	/* The maximum reported scroll position will be (nMax - (nPage - 1)),
+	   so we need to set nMax to the full range. */
+	si.nMax = lptw->CharSize.x * length;
+	si.nPos = lptw->ScrollPos.x;
+	SetScrollInfo(lptw->hWndText, SB_HORZ, &si, TRUE);
 	ShowScrollBar(lptw->hWndText, SB_HORZ, TRUE);
     } else {
 	lptw->ScrollMax.x = 0;
@@ -450,8 +454,17 @@ UpdateScrollBars(LPTW lptw)
     if (length >= lptw->ScreenSize.y) {
 	lptw->ScrollMax.y = max(0, lptw->CharSize.y * length - lptw->ClientSize.y);
 	lptw->ScrollPos.y = min(lptw->ScrollPos.y, lptw->ScrollMax.y);
-	SetScrollRange(lptw->hWndText, SB_VERT, 0, lptw->ScrollMax.y, FALSE);
-	SetScrollPos(lptw->hWndText, SB_VERT, lptw->ScrollPos.y, TRUE);
+
+	/* update scroll bar page size, range and position */
+	si.cbSize = sizeof(SCROLLINFO);
+	si.fMask = SIF_PAGE | SIF_RANGE | SIF_POS;
+	si.nPage = lptw->ClientSize.y;
+	si.nMin = 0;
+	/* The maximum reported scroll position will be (nMax - (nPage - 1)),
+	   so we need to set nMax to the full range. */
+	si.nMax = lptw->CharSize.y * length;
+	si.nPos = lptw->ScrollPos.y;
+	SetScrollInfo(lptw->hWndText, SB_VERT, &si, TRUE);
 	ShowScrollBar(lptw->hWndText, SB_VERT, TRUE);
     } else {
 	lptw->ScrollMax.y = 0;
@@ -531,6 +544,9 @@ TextPutCh(LPTW lptw, BYTE ch)
 	case '\t': {
 	    uint tab = 8 - (lptw->CursorPos.x  % 8);
 	    sb_last_insert_str(&(lptw->ScreenBuffer), lptw->CursorPos.x, "        ", tab);
+	    UpdateText(lptw, tab);
+	    UpdateScrollBars(lptw);
+	    TextToCursor(lptw);
 	    break;
 	}
 	case 0x08:
@@ -567,7 +583,7 @@ TextPutStr(LPTW lptw, LPSTR str)
 
     while (*str) {
 	idx = lptw->CursorPos.x;
-	for (count = 0, n = 0; *str && (isprint(*str) || *str == '\t'); str++) {
+	for (count = 0, n = 0; *str && (isprint((unsigned char)*str) || (*str == '\t')); str++) {
 	    if (*str == '\t') {
 		uint tab;
 
@@ -592,7 +608,7 @@ TextPutStr(LPTW lptw, LPSTR str)
 	    NewLine(lptw);
 	    str++;
 	    n = 0;
-	} else if (*str && !isprint(*str) && *str != '\t') {
+	} else if (*str && !isprint((unsigned char)*str) && (*str != '\t')) {
 	    TextPutCh(lptw, *str++);
 	}
     }
@@ -679,6 +695,9 @@ DoLine(LPTW lptw, HDC hdc, int xpos, int ypos, int x, int y, int count)
     char *outp;
     LPLB lb;
 
+    /* silence compiler warning */
+    (void) num;
+
     idx = 0;
     num = count;
     if (y <= sb_length(&(lptw->ScreenBuffer))) {
@@ -706,7 +725,6 @@ DoLine(LPTW lptw, HDC hdc, int xpos, int ypos, int x, int y, int count)
     free(outp);
 #else
     while (num > 0) {
-	num = 0;
 	attr = *pa;
 	while ((num > 0) && (attr == *pa)) {
 	    /* skip over bytes with same attribute */
@@ -726,7 +744,7 @@ DoLine(LPTW lptw, HDC hdc, int xpos, int ypos, int x, int y, int count)
 	free(outp);
 
 	xpos += lptw->CharSize.x * (count - num - idx);
-	idx = count-num;
+	idx = count - num;
     }
 #endif
 }
@@ -964,44 +982,49 @@ TextMakeFont(LPTW lptw)
 
 
 static void
-TextSelectFont(LPTW lptw) {
+TextSelectFont(LPTW lptw)
+{
     LOGFONT lf;
     CHOOSEFONT cf;
     HDC hdc;
-    char lpszStyle[LF_FACESIZE];
-    LPSTR p;
+    LPTSTR p;
 
     /* Set all structure fields to zero. */
-    _fmemset(&cf, 0, sizeof(CHOOSEFONT));
-    _fmemset(&lf, 0, sizeof(LOGFONT));
+    memset(&cf, 0, sizeof(CHOOSEFONT));
+    memset(&lf, 0, sizeof(LOGFONT));
     cf.lStructSize = sizeof(CHOOSEFONT);
     cf.hwndOwner = lptw->hWndParent;
-    _fstrncpy(lf.lfFaceName,lptw->fontname,LF_FACESIZE);
-    if ( (p = _fstrstr(lptw->fontname," Bold")) != (LPSTR)NULL ) {
-	_fstrncpy(lpszStyle,p+1,LF_FACESIZE);
-	lf.lfFaceName[ (unsigned int)(p-lptw->fontname) ] = '\0';
+    _tcsncpy(lf.lfFaceName, lptw->fontname, LF_FACESIZE);
+    if ((p = _tcsstr(lptw->fontname, TEXT(" Bold"))) != NULL) {
+	lf.lfWeight = FW_BOLD;
+	lf.lfFaceName[p - lptw->fontname] = NUL;
+    } else {
+	lf.lfWeight = FW_NORMAL;
     }
-    else if ( (p = _fstrstr(lptw->fontname," Italic")) != (LPSTR)NULL ) {
-	_fstrncpy(lpszStyle,p+1,LF_FACESIZE);
-	lf.lfFaceName[ (unsigned int)(p-lptw->fontname) ] = '\0';
-    } else
-	_fstrcpy(lpszStyle,"Regular");
-    cf.lpszStyle = lpszStyle;
+    if ((p = _tcsstr(lptw->fontname, TEXT(" Italic"))) != NULL) {
+	lf.lfItalic = TRUE;
+	lf.lfFaceName[p - lptw->fontname] = NUL;
+    } else {
+	lf.lfItalic = FALSE;
+    }
+    lf.lfCharSet = DEFAULT_CHARSET;
     hdc = GetDC(lptw->hWndText);
     lf.lfHeight = -MulDiv(lptw->fontsize, GetDeviceCaps(hdc, LOGPIXELSY), 72);
     ReleaseDC(lptw->hWndText, hdc);
     lf.lfPitchAndFamily = FIXED_PITCH;
     cf.lpLogFont = &lf;
     cf.nFontType = SCREEN_FONTTYPE;
-    cf.Flags = CF_SCREENFONTS | CF_FIXEDPITCHONLY | CF_INITTOLOGFONTSTRUCT | CF_USESTYLE;
+    cf.Flags = CF_SCREENFONTS | CF_FIXEDPITCHONLY | CF_INITTOLOGFONTSTRUCT | CF_SCALABLEONLY;
+
     if (ChooseFont(&cf)) {
 	RECT rect;
-	_fstrcpy(lptw->fontname,lf.lfFaceName);
+
+	_tcscpy(lptw->fontname, lf.lfFaceName);
 	lptw->fontsize = cf.iPointSize / 10;
 	if (cf.nFontType & BOLD_FONTTYPE)
-	    lstrcat(lptw->fontname," Bold");
+	    _tcscat(lptw->fontname, TEXT(" Bold"));
 	if (cf.nFontType & ITALIC_FONTTYPE)
-	    lstrcat(lptw->fontname," Italic");
+	    _tcscat(lptw->fontname, TEXT(" Italic"));
 	TextMakeFont(lptw);
 	/* force a window update */
 	GetClientRect(lptw->hWndText, (LPRECT) &rect);
@@ -1420,9 +1443,8 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 			lptw->KeyBufIn = lptw->KeyBuf;	/* wrap around */
 		}
 	    }
-	    break;
+		break;
 	    case VK_CANCEL:
-		/* FIXME: Currently, this only supports interrupting fit. */
 		ctrlc_flag = TRUE;
 		break;
 	    } /* switch(wparam) */
@@ -1440,10 +1462,18 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetKeyState(VK_CONTROL) < 0) {
 	    switch(wParam) {
 	    case VK_INSERT:
-	    case 'C':
 		/* Ctrl-Insert: copy to clipboard */
 		SendMessage(lptw->hWndText, WM_COMMAND, M_COPY_CLIP, (LPARAM)0);
 		break;
+	    case 'C':
+		/* Ctrl-C: copy to clipboard, if there's selected text,
+		           otherwise indicate the Ctrl-C (break) flag */
+		if ((lptw->MarkBegin.x != lptw->MarkEnd.x) ||
+		    (lptw->MarkBegin.y != lptw->MarkEnd.y))
+		    SendMessage(lptw->hWndText, WM_COMMAND, M_COPY_CLIP, (LPARAM)0);
+		else
+		    ctrlc_flag = TRUE;
+		break;
 	    case 'V':
 		/* Ctrl-V: paste clipboard */
 		SendMessage(lptw->hWndText, WM_COMMAND, M_PASTE, (LPARAM)0);
@@ -1882,6 +1912,26 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 }
 
 
+void
+TextStartEditing(LPTW lptw)
+{
+    TextToCursor(lptw);
+    if (lptw->bFocus && !lptw->bGetCh) {
+	UpdateCaretPos(lptw);
+	ShowCaret(lptw->hWndText);
+    }
+    lptw->bGetCh = TRUE;
+}
+
+
+void
+TextStopEditing(LPTW lptw)
+{
+    if (lptw->bFocus && lptw->bGetCh)
+	HideCaret(lptw->hWndText);
+    lptw->bGetCh = FALSE;
+}
+
 /* ================================== */
 /* replacement stdio routines */
 
@@ -1899,12 +1949,8 @@ int WDPROC
 TextGetCh(LPTW lptw)
 {
     int ch;
-    TextToCursor(lptw);
-    lptw->bGetCh = TRUE;
-    if (lptw->bFocus) {
-	UpdateCaretPos(lptw);
-	ShowCaret(lptw->hWndText);
-    }
+
+    TextStartEditing(lptw);
     while (!TextKBHit(lptw)) {
 	/* CMW: can't use TextMessage here as it does not idle properly */
 	MSG msg;
@@ -1913,13 +1959,11 @@ TextGetCh(LPTW lptw)
 	DispatchMessage(&msg);
     }
     ch = *lptw->KeyBufOut++;
-    if (ch=='\r')
+    if (ch == '\r')
 	ch = '\n';
     if (lptw->KeyBufOut - lptw->KeyBuf >= lptw->KeyBufSize)
 	lptw->KeyBufOut = lptw->KeyBuf;	/* wrap around */
-    if (lptw->bFocus)
-	HideCaret(lptw->hWndText);
-    lptw->bGetCh = FALSE;
+    TextStopEditing(lptw);
     return ch;
 }
 
@@ -1940,12 +1984,12 @@ TextGetS(LPTW lptw, LPSTR str, unsigned int size)
 {
     LPSTR next = str;
 
-    while (--size>0) {
-	switch(*next = TextGetChE(lptw)) {
+    while (--size > 0) {
+	switch (*next = TextGetChE(lptw)) {
 	case EOF:
 	    *next = 0;
 	    if (next == str)
-		return (LPSTR) NULL;
+		return NULL;
 	    return str;
 	case '\n':
 	    *(next+1) = 0;
@@ -1968,7 +2012,7 @@ int WDPROC
 TextPutS(LPTW lptw, LPSTR str)
 {
     TextPutStr(lptw, str);
-    return str[_fstrlen(str)-1];
+    return str[strlen(str) - 1];
 }
 
 
